"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.exec = exports.Client = void 0;
const ssh2_1 = require("ssh2");
const event_1 = require("@zodash/event");
class Client {
    constructor(config) {
        this.config = config;
        this.client = new ssh2_1.Client();
        this._isReady = false;
        //
    }
    ensureReady() {
        if (!this._isReady) {
            throw new Error(`SSH is not ready`);
        }
    }
    get isReady() {
        return this._isReady;
    }
    connect(config) {
        const { client } = this;
        const _config = config !== null && config !== void 0 ? config : this.config;
        if (!_config) {
            throw new Error(`connect is required`);
        }
        return new Promise((resolve, reject) => {
            client
                .on('ready', () => {
                this._isReady = true;
                return resolve();
            })
                .on('error', (error) => {
                return reject(error);
            })
                .connect(_config);
        });
    }
    async close() {
        const { client } = this;
        this.ensureReady();
        client.end();
    }
    exec(command, options) {
        const { client } = this;
        this.ensureReady();
        return new Promise((resolve, reject) => {
            client.exec(command, options || {}, (err, stream) => {
                if (err) {
                    return reject(err);
                }
                const chunks = [];
                stream
                    .on('close', (code, signal) => {
                    const data = chunks.join('');
                    if (+code !== 0) {
                        return reject(new Error(data));
                    }
                    return resolve(data.trim());
                })
                    .on('data', (chunk) => {
                    chunks.push(chunk);
                })
                    .stderr.on('data', (chunk) => {
                    chunks.push(chunk);
                });
            });
        });
    }
    spawn(command, options) {
        const { client } = this;
        this.ensureReady();
        const emitter = new event_1.Event();
        client.shell(options || {}, (err, stream) => {
            if (err) {
                return emitter.emit('error', err);
            }
            stream
                .on('close', () => {
                this.client.end();
            })
                .on('data', (chunk) => {
                emitter.emit('data', chunk);
            })
                .stderr.on('data', (chunk) => {
                emitter.emit('data', chunk);
            });
            stream.end(command + '\n');
            emitter.close = emitter.destroy = () => stream.end('exit\n');
        });
        return emitter;
    }
}
exports.Client = Client;
async function exec(command, config, options) {
    const client = new Client(config);
    await client.connect();
    const response = await client.exec(command, options);
    await client.close();
    return response;
}
exports.exec = exec;
